#! /usr/bin/python
__author__ = "J Dean Brock"

from smbus import SMBus
import time
import json

RPI_I2C_BUS = 1

# I2C device addresses
BMP180ADDR = 0x77

BMP180_CHIPID_REG =         0xD0
BMP180_CHIPID_VALUE =       0x55

# Various constants
# obtained from bmp180.h
# in https://github.com/mozilla-b2g/kernel-android-galaxy-s2-ics

# Register addresses
BMP180_TAKE_MEAS_REG =      0xF4
BMP180_READ_MEAS_REG =      0xF6
BMP180_EEPROM_AC1 =         0xAA
BMP180_EEPROM_AC2 =         0xAC
BMP180_EEPROM_AC3 =         0xAE
BMP180_EEPROM_AC4 =         0xB0
BMP180_EEPROM_AC5 =         0xB2
BMP180_EEPROM_AC6 =         0xB4
BMP180_EEPROM_B1 =          0xB6
BMP180_EEPROM_B2 =          0xB8
BMP180_EEPROM_MB =          0xBA
BMP180_EEPROM_MC =          0xBC
BMP180_EEPROM_MD =          0xBE

# constants for what we want to measure  (I have no idea...)
BMP180_MEAS_TEMP =              0x2E
BMP180_MEAS_PRESSURE =          0x34
BMP180_MEAS_PRESS_OVERSAMP_0 =  0x34
BMP180_MEAS_PRESS_OVERSAMP_1 =  0x74
BMP180_MEAS_PRESS_OVERSAMP_2 =  0xB4
BMP180_MEAS_PRESS_OVERSAMP_3 =  0xF4

# BMP180 in big-endian order
def readBEWord(bus, dev, addr):
    bew = bus.read_word_data(dev, addr) 
    bms = bew & 0xFF
    if bms >= 128:
        bms = bms - 256
    bls = (bew >> 8) & 0xFF
    return bms*256 + bls

def readUnsignedBEWord(bus, dev, addr):
    bew = bus.read_word_data(dev, addr) 
    bms = bew & 0xFF
    bls = (bew >> 8) & 0xFF
    return bms*256 + bls

oss = 0
def readBMP180temp():
    bmp180bus = SMBus(RPI_I2C_BUS)

    # Read the EEPROM configuration values
    ac1 = readBEWord(bmp180bus, BMP180ADDR, BMP180_EEPROM_AC1)
    ac2 = readBEWord(bmp180bus, BMP180ADDR, BMP180_EEPROM_AC2)
    ac3 = readBEWord(bmp180bus, BMP180ADDR, BMP180_EEPROM_AC3)
    ac4 = readUnsignedBEWord(bmp180bus, BMP180ADDR, BMP180_EEPROM_AC4)
    ac5 = readUnsignedBEWord(bmp180bus, BMP180ADDR, BMP180_EEPROM_AC5)
    ac6 = readUnsignedBEWord(bmp180bus, BMP180ADDR, BMP180_EEPROM_AC6)
    b1  = readBEWord(bmp180bus, BMP180ADDR, BMP180_EEPROM_B1)
    b2  = readBEWord(bmp180bus, BMP180ADDR, BMP180_EEPROM_B2)
    mb  = readBEWord(bmp180bus, BMP180ADDR, BMP180_EEPROM_MB)
    mc  = readBEWord(bmp180bus, BMP180ADDR, BMP180_EEPROM_MC)
    md  = readBEWord(bmp180bus, BMP180ADDR, BMP180_EEPROM_MD)

    # Start the read
    bmp180bus.write_byte_data(BMP180ADDR,
                              BMP180_TAKE_MEAS_REG, BMP180_MEAS_TEMP)
    # Supposed to wait 4.5ms
    time.sleep(0.01)
    ut  = readBEWord(bmp180bus, BMP180ADDR, BMP180_READ_MEAS_REG)
    x1 = (ut-ac6)*ac5/32768
    x2 = mc * 2048 / (x1 + md)
    b5 = x1 + x2
    t = (b5 + 8) / 16
    c = t * 0.1

    # Start the read
    bmp180bus.write_byte_data(BMP180ADDR,
                              BMP180_TAKE_MEAS_REG,
                              BMP180_MEAS_PRESSURE + (oss<<6))
    # Supposed to wait 4.5ms
    time.sleep(0.007*(oss+1))
    up0 = bmp180bus.read_byte_data(BMP180ADDR, BMP180_READ_MEAS_REG)
    up1 = bmp180bus.read_byte_data(BMP180ADDR, BMP180_READ_MEAS_REG+1)
    up2 = bmp180bus.read_byte_data(BMP180ADDR, BMP180_READ_MEAS_REG+2)
    up = ((up0<<16) + (up1<<8) + up0) >> (8-oss)
    b6 = b5 - 4000
    x1 = (b2 * (b6 * b6 / 4096))/2048
    x2 = ac2 * b6 / 2048
    x3 = x1 + x2
    b3 = (((ac1 * 4 + x3) << oss) + 2)/4
    x1 = ac3 * b6 / 8192
    x2 = (b1 * (b6 * b6 / 4096)) / 65536
    x3 = ((x1 + x2) + 2)/ 4
    b4 = ac4 * (x3 + 32768)/ 32768
    b7 = (up - b3) * (50000 >> oss)
    if (b7 < 0x80000000):
        p = (b7 * 2) / b4
    else:
        p = (b7 / b4) * 2
    x1 = (p / 256) * (p / 256) ;
    x1 = (x1 * 3038) / 65536 ;
    x2 = (-7357 * p) / 65536 ;
    p  = p + (x1 + x2 + 3791) / 16 ;
    return {'temperature':c, 'pressure':p}

def main():
    c = readBMP180temp()
    jsonret = json.dumps(c)
    print jsonret
    return

if __name__ == "__main__":
    main()
